//package xml;

import org.w3c.dom.*;
import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.Stack;
import java.io.PrintStream;
import java.net.*;


/**
*
* @date   September 26, 1998 ( Last Updated September 26, 1998 )
* @author H. Timucin Ozdemir
*
*/
public class 
ComponentsDocument extends BaseDocument implements XMLDebugFlags, ComponentsConstants {
   
   Hashtable cmaps;

   /** isExtracted? */
   boolean isExtracted = true;

   /** org.w3c.dom.Document */
   Document dom;

   /** */
   public ComponentsDocument(Document _dom) {
             cmaps = new Hashtable();
               dom = _dom;
   }// end of CONSTRUCTOR

   /** */
   public Enumeration getRegisteredComponents() { 
      if(isExtracted) extractFromDocument();
      return cmaps.elements(); 
   }// end of getRegisteredComponents()

   /** */
   void addComponentRecord(ComponentRecord _item) { cmaps.put(_item.getComponentName(),_item);}

   /** */
   public void extractFromDocument() { 
      if(DEBUG) System.out.println(" in extractFromDocument() ");
      isExtracted = false;
      if(dom!=null) {
         Element componentsRoot = dom.getDocumentElement();
         if( componentsRoot.getTagName().equals(ComponentsLabel) ) extractItems(componentsRoot);
      } else System.out.println(" Document is NULL1");
      if(DEBUG) System.out.println(" out of extractFromDocument() ");
   }// end of extractFromDocument()

   /** */
   final void extractItems(Element componentsRoot) {
     if(DEBUG) System.out.println(" in extractItems() ");
     NodeList componentList = ((Node)componentsRoot).getChildNodes();
     Node     current=null; 
     int      i=0;
     if( componentList!=null && componentList.getLength()>0) {
        while( i<componentList.getLength() ) {
           current = componentList.item(i++);
           if( current!=null && current.getNodeType()==org.w3c.dom.Node.ELEMENT_NODE &&
               ((Element)current).getTagName().equals(ComponentItemLabel)) 
               extractOneItem((Element)current);
        }// end of while
     }// end of if 
     if(DEBUG) System.out.println(" out of extractItems() ");
   }// end of extractItems()

   /** */
   final void extractOneItem(Element componentItem) {
     if(DEBUG) System.out.println(" in extractOneItem() ");
     NodeList fedNames = ((Node)componentItem).getChildNodes();
     NodeList fedNames1 ;
     Node     current=null; 
     Node     current2=null; 
     int      i=0;
     String   cmp_name    = getAttributeValue(componentItem,nameLabel);
     String   cmp_imgfile = null;
     String   cmp_filename = null;
     String   cmp_filepath = null;
     String   cmp_fileurl  = null;
     fileNameRec a=null;
     NodeList bList = componentItem.getChildNodes();
     if( bList!=null && bList.getLength()>0 ) {
       for(i=0;i<bList.getLength();i++) {
         current = bList.item(i);
         if( current!=null && current.getNodeType()==org.w3c.dom.Node.ELEMENT_NODE ) {
          if( ((Element)current).getTagName().equals(imgLabel)) 
             cmp_imgfile = getAttributeValue((Element)current,fileLabel);
          if( ((Element)current).getTagName().equals(fileNameLabel)) 
             a=extractFileName((Element)current);
         }
       }// end of for
     }// end of if
     if(a!=null) {
       cmp_filename = a.getFileName();
       cmp_filepath = a.getFilePath();
       cmp_fileurl  = a.getFileUrl();
     }
     if(DEBUG) {
       System.out.println(" cmp_filename "+cmp_filename );
       System.out.println(" cmp_filepath "+cmp_filepath );
       System.out.println(" cmp_fileurl  "+cmp_fileurl  );
     }// end of if
     if( cmp_name!=null && cmp_filename!=null ) {
         ComponentRecord cmpItem = new ComponentRecord(cmp_name,cmp_imgfile,
                                                       cmp_filename,cmp_filepath,
                                                       cmp_fileurl);
         if(DEBUG) System.out.println(" add "+cmp_name);
         addComponentRecord(cmpItem);
        }// end  of if
     if(DEBUG) System.out.println(" out of extractOneItem() ");
   }// end of extractOneItem()

   /** */
   final fileNameRec extractFileName( Element componentItem) {
     String cmp_filename = null; 
     String cmp_filepath = null;
     String cmp_fileurl  = null;
     if(DEBUG) System.out.println(" in extractFileName() ");
     Node     current=null; 
     int      i=0;
     NodeList bList = componentItem.getChildNodes();
     if( bList!=null && bList.getLength()>0 ) {
       for(i=0;i<bList.getLength();i++) {
         current = bList.item(i);
         if( current!=null && current.getNodeType()==org.w3c.dom.Node.ELEMENT_NODE ) {
          String tg = ((Element)current).getTagName(); 
          if(tg.equals(nameLabel)) cmp_filename=getTextNodeValue((Element)current);
          else if(tg.equals(pathLabel)) cmp_filepath=getTextNodeValue((Element)current);
          else if(tg.equals(urlLabel))  cmp_fileurl=getTextNodeValue((Element)current);
         }
       }// end of for
     }// end of if
     if(DEBUG) {
       System.out.println(" cmp_filename "+cmp_filename );
       System.out.println(" cmp_filepath "+cmp_filepath );
       System.out.println(" cmp_fileurl  "+cmp_fileurl  );
       System.out.println(" out of extractFileName() ");
     }// end of if
     return new fileNameRec(cmp_filename, cmp_filepath, cmp_fileurl);
   }// end of extractFileName()

   /** */
   public void print(PrintStream o)  { 
      o.println( "<"+ComponentsLabel+">");
      Enumeration e = cmaps.elements();
      while( e.hasMoreElements() ) {
         ComponentRecord imItem = (ComponentRecord) e.nextElement();
         if( imItem!=null ) imItem.print(o);
      }// end of while
      o.println( "</"+ComponentsLabel+">");
   }// end of print()

   class fileNameRec {
     String cmp_filename, cmp_filepath, cmp_fileurl;
     fileNameRec( String _cmp_filename, String _cmp_filepath, String _cmp_fileurl) {
       cmp_filename=_cmp_filename; cmp_filepath=_cmp_filepath; cmp_fileurl=_cmp_fileurl;
     } 
     public String getFileName() { return cmp_filename; }
     public String getFilePath() { return cmp_filepath; }
     public String getFileUrl()  { return cmp_fileurl; }
   }

}// end of ImRDocument
